"""Data loading module for statistical tables."""

from pulchra.data.loader import load_nco_stats, load_rotamer_library, get_data_path

__all__ = ["load_nco_stats", "load_rotamer_library", "get_data_path"]
